@@ -1,7 +1,7 @@
 /*
  * mainwindow.cpp
  *
- * Copyright 2002-2016 Vesa Halttunen
+ * Copyright 2002-2017 Vesa Halttunen
  *
  * This file is part of Tutka.
  *
@@ -187,8 +187,10 @@ MainWindow::MainWindow(Player *player, QWidget *parent) :
     connect(ui->actionTrackPaste, SIGNAL(triggered()), this, SLOT(pasteTrack()));
     connect(ui->actionTrackClear, SIGNAL(triggered()), this, SLOT(clearTrack()));
     connect(ui->actionTrackSelectAll, SIGNAL(triggered()), this, SLOT(selectAllTrack()));
-    connect(ui->actionTrackInsert, SIGNAL(triggered()), this, SLOT(insertTrack()));
-    connect(ui->actionTrackDelete, SIGNAL(triggered()), this, SLOT(deleteTrack()));
+    connect(ui->actionTrackInsertCurrentBlock, SIGNAL(triggered()), this, SLOT(insertTrackCurrentBlock()));
+    connect(ui->actionTrackInsertAllBlocks, SIGNAL(triggered()), this, SLOT(insertTrackAllBlocks()));
+    connect(ui->actionTrackDeleteCurrentBlock, SIGNAL(triggered()), this, SLOT(deleteTrackCurrentBlock()));
+    connect(ui->actionTrackDeleteAllBlocks, SIGNAL(triggered()), this, SLOT(deleteTrackAllBlocks()));
     connect(ui->actionTrackTranspose, SIGNAL(triggered()), transposeDialog, SLOT(showTrack()));
     connect(ui->actionTrackExpandShrink, SIGNAL(triggered()), expandShrinkDialog, SLOT(showTrack()));
     connect(ui->actionTrackChangeInstrument, SIGNAL(triggered()), changeInstrumentDialog, SLOT(showTrack()));
@@ -270,7 +272,12 @@ bool MainWindow::eventFilter(QObject *watched, QEvent *event)
 {
     bool handled = false;
 
-    if ((event->type() == QEvent::KeyPress || event->type() == QEvent::KeyRelease) && watched->isWindowType() && dynamic_cast<QLineEdit *>(QApplication::focusWidget()) == NULL && dynamic_cast<QSpinBox *>(QApplication::focusWidget()) == NULL && dynamic_cast<Tracker *>(QApplication::focusWidget()) == NULL && dynamic_cast<QComboBox *>(QApplication::focusWidget()) == NULL) {
+#if QT_VERSION >= QT_VERSION_CHECK(5, 0, 0)
+    bool isWindow = watched->isWindowType();
+#else
+    bool isWindow = watched == this || watched->inherits("QDialog");
+#endif
+    if ((event->type() == QEvent::KeyPress || event->type() == QEvent::KeyRelease) && isWindow && dynamic_cast<QLineEdit *>(QApplication::focusWidget()) == NULL && dynamic_cast<QSpinBox *>(QApplication::focusWidget()) == NULL && dynamic_cast<Tracker *>(QApplication::focusWidget()) == NULL && dynamic_cast<QComboBox *>(QApplication::focusWidget()) == NULL) {
         QKeyEvent *keyEvent = static_cast<QKeyEvent *>(event);
 
         if (QApplication::activeWindow() == this || (keyEvent->modifiers() != Qt::NoModifier && keyEvent->key() != Qt::Key_Backtab) || (keyEvent->key() != Qt::Key_Tab && keyEvent->key() != Qt::Key_Backtab && keyEvent->key() != Qt::Key_Left && keyEvent->key() != Qt::Key_Right && keyEvent->key() != Qt::Key_Home && keyEvent->key() != Qt::Key_End && keyEvent->key() != Qt::Key_PageUp && keyEvent->key() != Qt::Key_PageDown)) {
@@ -338,11 +345,11 @@ bool MainWindow::keyPress(QKeyEvent *event)
 #endif
         case Qt::Key_Left:
             // CTRL-Left: Previous instrument
-            if (ui->spinBoxInstrument->value() > 0) {
+            if (ui->spinBoxInstrument->value() > 1) {
                 ui->spinBoxInstrument->setValue(ui->spinBoxInstrument->value() - 1);
 
                 // Make sure the instrument exists
-                song->checkInstrument(ui->spinBoxInstrument->value());
+                song->checkInstrument(ui->spinBoxInstrument->value() - 1);
             }
             handled = true;
             break;
@@ -352,7 +359,7 @@ bool MainWindow::keyPress(QKeyEvent *event)
                 ui->spinBoxInstrument->setValue(ui->spinBoxInstrument->value() + 1);
 
                 // Make sure the instrument exists
-                song->checkInstrument(ui->spinBoxInstrument->value());
+                song->checkInstrument(ui->spinBoxInstrument->value() - 1);
             }
             handled = true;
             break;
@@ -552,10 +559,10 @@ bool MainWindow::keyPress(QKeyEvent *event)
 
         // Select an instrument if an instrument selection key was pressed
         if (instrument >= 0) {
-            ui->spinBoxInstrument->setValue(instrument);
+            ui->spinBoxInstrument->setValue(instrument + 1);
 
             // Make sure the instrument exists
-            song->checkInstrument(ui->spinBoxInstrument->value());
+            song->checkInstrument(instrument);
             handled = true;
         }
     }
@@ -582,6 +589,7 @@ void MainWindow::setSong(Song *song)
         disconnect(ui->actionSettingsSendMidiSync, SIGNAL(triggered(bool)), this->song, SLOT(setSendSync(bool)));
         disconnect(this->song, SIGNAL(nameChanged()), this, SLOT(setWindowTitle()));
         disconnect(this->song, SIGNAL(modifiedChanged()), this, SLOT(setWindowTitle()));
+        disconnect(this->song, SIGNAL(maxTracksChanged(uint)), this, SLOT(setDeleteTrackVisibility()));
     }
 
     this->song = song;
@@ -604,6 +612,7 @@ void MainWindow::setSong(Song *song)
     connect(ui->actionSettingsSendMidiSync, SIGNAL(triggered(bool)), song, SLOT(setSendSync(bool)));
     connect(this->song, SIGNAL(nameChanged()), this, SLOT(setWindowTitle()));
     connect(this->song, SIGNAL(modifiedChanged()), this, SLOT(setWindowTitle()));
+    connect(this->song, SIGNAL(maxTracksChanged(uint)), this, SLOT(setDeleteTrackVisibility()));
 }
 
 void MainWindow::setSection(unsigned int section)
@@ -690,14 +699,14 @@ void MainWindow::setTime(unsigned int time)
 
 void MainWindow::setInstrument(int instrument)
 {
-    if (instrument >= 0) {
+    if (instrument > 0) {
         Instrument *oldInstrument = song->instrument(this->instrument);
         if (oldInstrument != NULL) {
             disconnect(oldInstrument, SIGNAL(nameChanged(QString)), ui->lineEditInstrument, SLOT(setText(QString)));
             disconnect(ui->lineEditInstrument, SIGNAL(textChanged(QString)), oldInstrument, SLOT(setName(QString)));
         }
 
-        this->instrument = instrument;
+        this->instrument = instrument - 1;
         song->checkInstrument(this->instrument);
 
         Instrument *newInstrument = song->instrument(this->instrument);
@@ -707,7 +716,7 @@ void MainWindow::setInstrument(int instrument)
         ui->lineEditInstrument->blockSignals(true);
         ui->lineEditInstrument->setText(newInstrument->name());
         ui->lineEditInstrument->blockSignals(false);
-        ui->tracker->setInstrument(instrument + 1);
+        ui->tracker->setInstrument(instrument);
     }
 }
 
@@ -721,7 +730,7 @@ void MainWindow::setSelection(int startTrack, int startLine, int endTrack, int e
 
 void MainWindow::showAbout()
 {
-    QMessageBox::about(this, tr("About Tutka"), tr("Tutka 1.1.0\n\n© 2002-2016 Vesa Halttunen <vesuri@jormas.com>"));
+    QMessageBox::about(this, tr("About Tutka"), tr("Tutka 1.1.1\n\n© 2002-2017 Vesa Halttunen <vesuri@jormas.com>"));
 }
 
 void MainWindow::cutSelection()
@@ -823,16 +832,26 @@ void MainWindow::selectAllTrack()
     ui->tracker->setSelection(ui->tracker->track(), 0, ui->tracker->track(), song->block(block)->length() - 1);
 }
 
-void MainWindow::insertTrack()
+void MainWindow::insertTrackCurrentBlock()
 {
     song->block(block)->insertTrack(ui->tracker->track());
 }
 
-void MainWindow::deleteTrack()
+void MainWindow::insertTrackAllBlocks()
+{
+    song->insertTrack(ui->tracker->track());
+}
+
+void MainWindow::deleteTrackCurrentBlock()
 {
     song->block(block)->deleteTrack(ui->tracker->track());
 }
 
+void MainWindow::deleteTrackAllBlocks()
+{
+    song->deleteTrack(ui->tracker->track());
+}
+
 void MainWindow::setExternalSync()
 {
     player->setExternalSync(externalSyncActionGroup->checkedAction() == ui->actionExternalSyncOff ? Player::Off : Player::Midi);
@@ -912,7 +931,7 @@ void MainWindow::handleMidiInput(const QByteArray &data)
     case 0x90:
         // Note on
         if (ui->checkBoxEdit->isChecked()) {
-            block->setNote(ui->tracker->line(), ui->tracker->cursorTrack(), data[1] / 12, data[1] % 12 + 1, ui->spinBoxInstrument->value() + 1);
+            block->setNote(ui->tracker->line(), ui->tracker->cursorTrack(), data[1] / 12, data[1] % 12 + 1, ui->spinBoxInstrument->value());
             block->setCommandFull(ui->tracker->line(), ui->tracker->cursorTrack(), ui->tracker->commandPage(), Player::CommandVelocity, data[2]);
             if (ui->checkBoxChord->isChecked()) {
                 ui->tracker->addChordNote();
@@ -965,7 +984,8 @@ void MainWindow::setSongPath(const QString &path)
 void MainWindow::setDeleteTrackVisibility()
 {
     if (song != NULL && block < song->blocks()) {
-        ui->actionTrackDelete->setEnabled(song->block(block)->tracks() > 1);
+        ui->actionTrackDeleteCurrentBlock->setEnabled(song->block(block)->tracks() > 1);
+        ui->actionTrackDeleteAllBlocks->setEnabled(song->maxTracks() > 1);
     }
 }
 
@@ -1061,5 +1081,5 @@ void MainWindow::advancePlayerBySpaceLines()
 
 void MainWindow::playPressedNote(unsigned char note)
 {
-    player->playNote(ui->spinBoxInstrument->value(), ui->comboBoxKeyboardOctaves->currentIndex() * 12 + note, 127, ui->tracker->cursorTrack());
+    player->playNote(ui->spinBoxInstrument->value() - 1, ui->comboBoxKeyboardOctaves->currentIndex() * 12 + note, 127, ui->tracker->cursorTrack());
 }
