@@ -1,7 +1,7 @@
 /*
  * mainwindow.h
  *
- * Copyright 2002-2016 Vesa Halttunen
+ * Copyright 2002-2017 Vesa Halttunen
  *
  * This file is part of Tutka.
  *
@@ -91,8 +91,10 @@ private slots:
     void pasteTrack();
     void clearTrack();
     void selectAllTrack();
-    void insertTrack();
-    void deleteTrack();
+    void insertTrackCurrentBlock();
+    void insertTrackAllBlocks();
+    void deleteTrackCurrentBlock();
+    void deleteTrackAllBlocks();
     void setExternalSync();
     void save();
     void saveAs();
